/*
 *  Kylin Box security
 *
 *  This file contains apis for Kylin Box security libraries
 *
 *  Authors:  songchuanfei <songchuanfei@kylinos.cn>
 *  Authors:  huanfuxi <huanfuxi@kylinos.cn>
 *  Authors:  mengyuan <mengyuan@kylinos.cn>
 *
 *  Copyright (C) 2017,Tianjin KYLIN Information Technology Co., Ltd.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the
 *  Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 *  Boston, MA 02110-1301, USA.
 */
#ifndef BOX_LIB_INCLUDE_H
#define BOX_LIB_INCLUDE_H

#include <sys/acl.h>
#include <linux/limits.h>

#ifdef __cplusplus
extern "C"{
#endif

#define BOX_STATUS_ENTRY "/sys/kernel/security/box/status"
#define BOX_ROOT_ENTRY "/home"
#define BOX_USER_ENTRY BOX_ROOT_ENTRY "/%s"
#define BOX_USER_ENTRY_NEXT BOX_USER_ENTRY "/%s"
#define BOX_NORMAL_ENTRY BOX_USER_ENTRY "/%s"
#define BOX_NORMAL_ENTRY_NEXT BOX_NORMAL_ENTRY "/%s"
#define BOX_ENCRYPT_ENTRY BOX_USER_ENTRY "/%s"
#define BOX_ENCRYPT_ENTRY_NEXT BOX_ENCRYPT_ENTRY  "/%s"
#define BOX_ENCRYPT_CONF BOX_ENCRYPT_ENTRY_NEXT "/.box_ecrypt.conf"
#define BOX_ENCRYPT_DATA BOX_ENCRYPT_ENTRY_NEXT "/.data"
//#define BOX_ENCRYPT_DEFAULT_MOUNTPOINT  BOX_ENCRYPT_ENTRY "/mounted/%s"
#define BOX_ENCRYPT_DEFAULT_MOUNTPOINT  BOX_ENCRYPT_ENTRY_NEXT
#define BOX_WATCHER "/tmp/box-watcher"

#define BOX_NAME "box"
#define BOX_ROOT_HOMT_PATH  "/root"
#define ROOT_BOX_PATH "/root/box"
#define ROOT_NORMAL_ENRTY ROOT_BOX_PATH "/%s"
#define ROOT_ENCRYPT_CONF ROOT_NORMAL_ENRTY "/.box_ecrypt.conf"
#define ROOT_ENCRYPT_DATA ROOT_NORMAL_ENRTY "/.data"


/* 
 * 加密方案参数
 */
#define NORMAL          0X00000000 
#define ECRYPTFS        0X01000000

#define AES256          0X00010000
#define AES128          0X00020000
#define AES196          0X00030000
#define BLOWFISH128     0X00040000
#define BLOWFISH256     0X00050000
#define BLOWFISH448     0X00060000
#define TWOFISH192      0X00070000



/* 
 * box 类型,用于搜索
 */
#define BOX_NORMAL      0b00000001
#define BOX_ENCRYPTED   0b00000010
#define BOX_SHARED      0b00000100
#define BOX_UNSHARED    0b00001000
#define BOX_SELF        0b00010000
#define BOX_OTHERS      0b00100000
#define BOX_MOUNTED     0b01000000
#define BOX_NOTMOUNTED  0b10000000
#define BOX_ALL         (BOX_NORMAL|BOX_ENCRYPTED|BOX_SHARED|BOX_UNSHARED|BOX_SELF|BOX_OTHERS|BOX_MOUNTED|BOX_NOTMOUNTED)

/* 
 * errno扩展
 */
#define EBOXDISABLED 500        /* BOX is DISABLED */
#define EINVALUSERNAME 501      /* USERNAME is invalid */
#define EUSERNAMETOOLONG 502    /* USERNAME is too long */
#define EUNINITIALIZED  503     /* BOX is NOT initialized */
#define ECONFBROKEN 504         /* conf is broken */
#define EWRITECONF  505         /* writing conf failed */
#define EMOUNTEDCONF 506        /* mountpoints conf broken */
#define EKEYTOOLONG 507        /* passphrase is too long */
#define EFNEK_SUPPORT 508       /* kernel does not support filename encryption */
#define EBOXISMOUNTED 509       /* box is already mounted */
#define EBOXISENCRYPTED 510     /* box is already encrypted */
#define EDELDIRFAILED 511       /* del files failed */
#define EMOUNTED    512         /* mounted failed */
#define EMOVEDATA   513         /* moving date failed */
#define EUMOUNT     514         /* umount failed */
#define ENOTMOUNT   515         /* not mounted */
#define ENOTENCRYPTED 516       /* box is not encrypted */ 
#define EISMOUNTED  517         /* box is mounted */
#define EINVALNAME  518         /* invalid user name */
#define EPASSWORD   519         /* password is incorrect */
#define EINVALBOXNAME 520       /* invalid box name*/
#define EINVALPASS  521         /* invalid passphrase */
#define NONSUPPORT  522         /* 暂时不支持root*/
#define FILE_EXIS_ERR 17        /* 解决界面创建保存问题*/

#define SIGSIZE 16              /* 与ecryptfs一致 */


#define ECRYPT_CHECK_ON 1
#define ECRYPT_CHECK_OFF 0


/* 
 * @crypto_algorithm:加密算法
 * @password:密码,需要为字符串形式，末位'\0'会排除（ecryptfs要求）
 * ,后期可能会变更，支持key,file等形式
 * @sig:密码哈希值
 */
struct passphrase{
    void *password;
    size_t pass_size;
};
struct box_crypto{
    unsigned long crypto_algorithm;
    struct passphrase password;
    char sig[SIGSIZE+1];
};

/* struct box_mounted_info_list{
    id_t mounted_uid;
    struct box_mounted_info_list *next;
};
 */
struct box_info{
    char *name;
    char *path;
    char *owner_name;
    acl_t acl;
    unsigned char box_type;
    /* struct box_crypto *crypt_info; */
    /* struct box_mounted_info_list *mounted_info; */
};

struct box_info_list{
    struct box_info info;
    struct box_info_list *next;
};


/* 为当前用户创建box
 */
int box_create(const char *box_name);

/* 删除当前用户的box
 */
int box_delete(const char *box_name);

/* 为当前用户box改名 */
int box_change_name(const char *old_box_name,const char *new_box_name);

/* 
 * 共享操作
 * perm,id,id_type均为ACL中的定义
 * 操作都是基于当前用户创建的box
 */
int box_add_share(const char *box_name, mode_t perm, id_t id, int id_type);
int box_remove_share(const char *box_name, id_t id, int id_type);
int box_clear_share(const char *box_name);

/* 
 * 加密操作
 * @box_crypto_create为创建加密box,如果box已存在则对已有文件进行加密；如果已加密则返回错误
 */
int box_crypto_create(const char *box_name,struct box_crypto *crypto, int action);
int box_crypto_change_passpharse(const char *box_name,const struct passphrase *old_password,struct box_crypto *new_crypto);

/*解密box*/
int box_crypto_decrypt(const char *box_name,const struct passphrase *password);
int box_crypto_verify_passphrase(const char *box_name,const struct passphrase *password);

/* 
 * 挂载操作
 * 对username下的box_name进行挂载（卸载），原地挂载
 * del /box/<owner's username>/<boxname>/mounted/<username who mounted> del
 */
int box_crypto_mount(const char *ownername,const char *box_name,const struct passphrase *password);
int box_crypto_umount(const char *ownername,const char *box_name);
int box_crypto_umount_all(const char *ownername);

int box_check_owner(const char *ownername);
int check_box_exist(const char *box_name);
int box_crypto_verify(const char *box_name);

int box_enabled();
int box_get_info(const char *ownername,const char *box_name,struct box_info **info);
int box_get_list(int box_type,struct box_info_list **list);

int box_init();
int box_destroy();
int box_crypto_destroy(struct box_crypto *crypt);
int box_info_destroy(struct box_info *info);
int box_info_list_destroy(struct box_info_list *list);
int box_passphrase_destroy(struct passphrase *password);


/* 
 * 对错误信息进行解析，返回字符串
 * @为错误号，>=0
 */
char *box_err(int err);

#ifdef __cplusplus
}
#endif
#endif

